/*
 *  Copyright (C) 2003 Hiroyuki Ikezoe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <unistd.h>
#include <libxml/parser.h>

#include "gtkmozembed.h"
#include "kazehakase-type.h"
#include "prefs.h"
#include "rssbar.h"
static void kztoolbar_start_element(void *ctx , const xmlChar *name, const xmlChar **atts);
static void kztoolbar_file_load (void);
static void kztoolbar_rss_menu_new(Kazehakase *kz, GtkWidget* rssbar, RSS *rss);
static void kztoolbar_update(void);

KazehakaseToolbar *kztoolbar;

/* create new rssbar */
GtkWidget *kztoolbar_new (Kazehakase *kz)
{
	kztoolbar = g_new0(KazehakaseToolbar,1);
	
	kztoolbar->parent_kazehakase = kz;
	kztoolbar->items = g_slist_alloc();
	
	kztoolbar->toolbar =  gtk_toolbar_new();
	
	/* load toolbar config file */
	kztoolbar_file_load();
	if (!g_thread_supported())
	  g_thread_init(NULL);
	
	kztoolbar_update();
	
	return kztoolbar->toolbar;
}

static void kztoolbar_file_load (void)
{
	static xmlSAXHandler parser = 
	{
		NULL, /* internalSubset */
		NULL, /* isStandalone */
		NULL, /* hasInternalSubset */
		NULL, /* hasExternalSubset */
		NULL, /* resolveEntity */
		NULL, /* getEntity */
		NULL, /* entityDecl */
		NULL, /* notationDecl */
		NULL, /* attributeDecl */
		NULL, /* elementDecl */
		NULL, /* unparsedEntityDecl */
		NULL, /* setDocumentLocator */
		NULL, /* startDocument */
		NULL, /* endDocument */
		(startElementSAXFunc) kztoolbar_start_element, /* startElement */
		NULL, /* endElement */
		NULL, /* reference */
		NULL, /* characters */
		NULL, /* ignorableWhitespace */
		NULL, /* processingInstruction */
		NULL, /* comment */
		NULL, /* xmlParserWarning */
		NULL, /* xmlParserError */
		NULL, /* xmlParserError */
		NULL, /* getParameterEntity */
		NULL, /* cdataBlock; */
		NULL,  /* externalSubset; */
		1
	};
	
	/* this codes from galeon-1.2.7 */
        gchar *toolbarfile;

	/* build the filename */
        toolbarfile = g_strconcat (g_get_home_dir(), "/.kazehakase/toolbar.xml", NULL);
	
	/* load the file if we can access it */
	if (access (toolbarfile, F_OK) != -1)
	{
		xmlSAXParseFile (&parser, toolbarfile, TRUE);
	}
	g_free(toolbarfile);
}

static void
kztoolbar_start_element(void *ctx , const xmlChar *name, const xmlChar **atts)
{
	/* parse element */
	if (xmlStrcmp (name, "rss") == 0 && atts != NULL)
	{
		/* <rss> element */
		rssbar_parse_site_element (atts, kztoolbar);
	}
	else
	{
		g_warning ("unknown element '%s' in toolbar.xml\n", name);
	}

}

static void kztoolbar_update(void)
{
	GSList *items;
	RSS *item;
	guint item_num, n;
	GThread *thread;
	
	items = kztoolbar->items;
	item_num = g_slist_length(kztoolbar->items);
	for (n = 1; n < item_num; n++)
	{
		item = (RSS*) g_slist_nth_data(items, n);
		thread = g_thread_create((void *)&rss_menu_update, item, FALSE, NULL);
	}	
}
